#include "audio.h"

#include<unistd.h>
#include<fcntl.h>
#include<sys/types.h>
#include<sys/stat.h>
#include<linux/soundcard.h>

/* File description of sound card device */
static int fd;

/******************************************************************//*
 * Open sound card device
 * parameter
 *        dev_pathname: Pathname of sound device path
 *
 * return
 *        Nonnegative number represent open successful,
 *        and negative number represent faild
 ******************************************************************/
int sc_open(const char *pathname)
{
	/* Open sound device */
	fd = open(pathname, O_WRONLY);
	if (fd < 0) {
		perror("Error occured on open sound device");
		return -1;
	}
}

/******************************************************************//*
 * Config sound device
 * parameter
 *        sample_bits: sample bits
 *        sample_rate: sample freq
 *
 * return
 *        Nonnegative number represent configure successful,
 *        and negative number represent faild
 ******************************************************************/
int sc_config(const int sample_bits,
			  const int sample_rate)
{
	/* Setting sample bits of sound device */ 
	if (ioctl(fd, SOUND_PCM_WRITE_BITS, &sample_bits) == -1) {
		perror("Error occured on SOUND_PCM_WRITE_BITS ioctl");
		return -1;
	}

	/* Setting sample rate of sound device */
	if (ioctl(fd, SOUND_PCM_WRITE_RATE, &sample_rate) == -1) {
		perror("Error occured on SOUND_PCM_WRITE_RATE ioctl");
		return -1;
	}
}

/******************************************************************//*
 * Play specify file
 * parameter
 *        filename: The sound file that will be played
 *
 * return
 *        Nonnegative number represent played successful,
 *        and negative number represent faild
 ******************************************************************/
int sc_playsound(const char *filename)
{
	/* Fild description of sound file */
	static int file;

	struct stat stat_buf;
	unsigned char *buf = NULL;

	/* Open sound file */
	file = open(filename, O_RDONLY);
	if (file < 0) {
		perror("Error occured on open sound file");
		return -1;
	}

	/* Get file's status */
	if (fstat(file, &stat_buf)) {
		perror("Error occured on get sound file status");
		return -1;
	}
	if (!stat_buf.st_size) {
		printf("Invalid sound file\n");
		close(file);
		return -1;
	}

	/* Allocate memory for store sound's data */
	buf = malloc(stat_buf.st_size);
	if (!buf) {
		printf("Allocate memory for sound faild\n");
		close(file);
		return -1;
	}

	/* Read data from sound's file */
	if (read(file, buf, stat_buf.st_size) < 0) {
		perror("Fail to read sound file");
		free(buf);
		close(file);
		return -1;
	}

	/* Wrate sound's data to sound device */
	if (write(fd, buf, stat_buf.st_size) == -1) {
		perror("Fail to play sound");
		return -1;
	}

	free(buf);
	close(file);

	return 0;
}

/******************************************************************//*
 * Unuse sound card and free resource
 * parameter
 *        None
 *
 * return
 *        None
 ******************************************************************/
void sc_free(void)
{
	close(fd);
}
