#ifndef _LCD_H_
#define _LCD_H_

#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <linux/fb.h>
#include <sys/mman.h>
#include <stdlib.h>

typedef enum {FAILD = 0, SUCCESS } RET_STATUS;

/*************************************************************//*
 * Initialize lcd
 * parameter
 *         dev_pathname: The pathname of LCD device node
 *
 * return
 *         Return initialize status, SUCCESS represent initialize
 *         success and FAILD represent faild
 ************************************************************/
extern RET_STATUS init_lcd(const char *dev_pathname);

/*************************************************************//*
 * Print information of LCD that has been initialize
 * parameter
 *         None
 *
 * return
 *         None
 ************************************************************/
extern void print_lcdinfo(void);

/*************************************************************//*
 * Draw on LCD
 * parameter
 *         start: The start position on LCD
 *         end:   The end position on LCD
 *         color: The value of color
 *
 * return
 *         None
 ************************************************************/
extern void draw_region(const long int start,
						const long int end,
						const short color);

/*************************************************************//*
 * Draw a rectangle on LCD
 * parameter
 *         start_x: The start point x position on LCD
 *         start_y: The start point y position on LCD
 *         end_x:   The end point x position on LCD
 *         end_y:   The end point y position on LCD
 *
 * return
 *         None
 ************************************************************/
extern void draw_rectangle(long int start_x,
					 	    long int start_y,
						    const long int end_x,
						    const long int end_y,
						    const short color);

/*************************************************************//*
 * Restitute LCD and close the file description
 * parameter
 *         None
 *
 * return
 *         None
 ************************************************************/
void restitute_lcd(void);

#endif
