#ifndef _TWI_H_
#define _TWI_H_

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

typedef enum { FAILD = 0, SUCCESS } RET_STATUS;

/**************************************************************//*
 * Initialize i2c device
 * parameter
 *         dev_pathname: The pathname of i2c device
 *         slave_addr  : Address of slave device
 *
 * return
 *         Return the status of initialize, FAILD represent
 *         initialize hasn't complate and SUCCESS represent
 *         successful
 ***************************************************************/
RET_STATUS i2c_init(const char *dev_pathname,
					long slave_addr);

/**************************************************************//*
 * Write data to i2c slave device
 * parameter
 *         fd      : The file description of i2c device
 *         send_buf: The data that will be writed to slave device
 *         addr    : The address that start position of writing
 *         num     : The number of bytes will be writed that in
 *                   the send_buf
 *
 * return
 *         Return the status of writing, FAILD represent writing
 *         hasn't complate and SUCCESS represent successful
 ***************************************************************/
RET_STATUS i2c_write(const char *send_buf,
					 int addr,
					 int *num);


/**************************************************************//*
 * Read data from i2c slave device
 * parameter
 *         fd      : The file description of i2c device
 *         recv_buf: It store data that will be read from i2c
 *                   slave device
 *         addr    : The address that start position of reading
 *         num     : The number of bytes will be readed
 *
 * return
 *         Return the status of reading, FAILD represent reading
 *         hasn't complate and SUCCESS represent successful
 ***************************************************************/
RET_STATUS i2c_read(char *recv_buf,
					int addr,
					const int buf_size,
					int *num);

/**************************************************************//*
 * Deinitialize i2c device
 * parameter
 *         None
 *
 * return
 *         None
 ***************************************************************/
void i2c_deinit(void);

#endif
