#include "rs485.h"

#define BAUD_RATE B2400
#define BUF_SIZE  128 
#define END_FLAG  0x23

/***************************************************************//*
 * Test rs485 function 
 * parameter
 *		none
 *
 * return
 *		none
 ***************************************************************/
void rs485_test(void);

/***************************************************************//*
 * Receive thread 
 * parameter
 * 		none
 *
 * return
 *		none
 ***************************************************************/
void recv_thread(void);

/*************************************************************//*
 * Main function
 * parameter
 *		argc: The number of parameters,
 *		argv: The first of parameters is represent this program's
 *			  name, the second of parameters is represent runing
 *			  mode(if no null), and the third of parameters is 
 *			  reprsent the device path name that will be used,
 *            default is use /dev/ttyS1
 * return
 *		Return the status's code the program
n **************************************************************/
int main(int argc, char *argv[])
{
	char *tty_dev = "/dev/ttyS1"; // Default use ttyS1

	/* Get device node from input parameter */
	if (argc > 1) {
		tty_dev = argv[1];
	}

	/* Initialize device */
	if (rs485_init(tty_dev, BAUD_RATE) < 0) {
		printf("Exit now\n");
		exit(EXIT_FAILURE);
	}
	printf("Initialize %s success\n", tty_dev);

	/* Enable rs485 mode */
	if (rs485_enable(ENABLE) == -1) {
		printf("Exit now\n");
		exit(EXIT_FAILURE);
	}
	printf("RS485 mode enable\n");

	/* Test rs485 function */
	rs485_test();

	/* Exit rs485 */
	rs485_exit();
	
	return EXIT_SUCCESS;
}

/***************************************************************//*
 * Test rs485 function 
 * parameter
 *		mode: The runing mode, 'm' or 'M' represent master mode,
 * 			  and the other charactor represent slave mode
 *
 * return
 *		none
 ***************************************************************/
void rs485_test(void)
{
	char tx_buf[BUF_SIZE] = { 0 };
	pthread_t id;
	ssize_t res;

	/* Create a thread for receive message */
	pthread_create(&id, NULL, (void*)recv_thread, NULL);
	
	/* Send message */
	while (1) {
		memset(tx_buf, 0, BUF_SIZE);
		gets(tx_buf);
		tx_buf[strlen(tx_buf)] = 0x23;
		if ((res = rs485_write(tx_buf, strlen(tx_buf))) > 0) { // Send to slave
			tx_buf[strlen(tx_buf) - 1] = '\0';
			printf("Send %d bytes msg: %s\n", res - 1, tx_buf);
		}
	}
}

/***************************************************************//*
 * Receive thread 
 * parameter
 * 		none
 *
 * return
 *		none
 ***************************************************************/
void recv_thread(void)
{
	char rx_buf[BUF_SIZE] = { 0 };
	int  msg_ind = 0;;
	int  res;

	while (1) {
		if ((res = rs485_read(&rx_buf[msg_ind], BUF_SIZE - msg_ind - 1)) > 0) {
			msg_ind += res;
			if ((rx_buf[msg_ind - 1] == END_FLAG) ||
				(msg_ind == (BUF_SIZE - 1))) {
				rx_buf[msg_ind - 1] = '\0';
				printf("Receive %d bytes msg: %s\n", msg_ind - 1, rx_buf);
				memset(rx_buf, 0, BUF_SIZE);
				msg_ind = 0;
			}
		}
	}
}
