#ifndef _RS485_H_
#define _RS485_H_

#include "string.h"
#include "unistd.h"
#include "fcntl.h"
#include "errno.h"
#include "sys/types.h"
#include "sys/stat.h"
#include "sys/ioctl.h"
#include "stdlib.h"
#include "stdarg.h"
#include "stdio.h"
#include "termios.h"
#include "linux/serial.h"

typedef enum {DISABLE = 0, ENABLE} RS485_ENABLE_t;

/* Device file description */
extern int fd;

/***************************************************************//*
 * Initialize rs485
 * parameter
 *		pathname: The device will be opened
 *		speed: The baudrate will be setted 
 *
 * return
 *		If initialize success, return file description of the device
 *		otherwise return a negative number
 ****************************************************************/
extern int rs485_init(const char *pathname, const speed_t speed);

/***************************************************************//*
 * Enable or Disable rs485 mode 
 * parameter
 *		enable: The flag of enable of disable rs485 mode
 *
 * return
 *		The result of setting, if setting occur error,return 
 *		a negative number
 ***************************************************************/
extern int rs485_enable(const RS485_ENABLE_t enable);

/***************************************************************//*
 * Write data to rs485 
 * parameter
 *		tx_buf: The content of writing
 *		num: How many bytes of in tx_buf will be writed
 *
 * return
 *		The number of the write bytes, if writting occur error,
 *		return -1
 **************************************************************/
extern ssize_t rs485_write(const char *tx_buf, const ssize_t num);

/***************************************************************//*
 * Read data from rs485
 * parameter
 *		rx_buf: Store the reading content
 *		num: How many bytes will be readed
 * return
 *		The number of reading bytes, if reading occur error,
 *		return -1
 **************************************************************/
extern ssize_t rs485_read(char *rx_buf, const ssize_t num);

/**************************************************************//*
 * Close fd when exit rs485
 * parameter
 *      None
 *
 * return
 *      None
 *************************************************************/
extern void rs485_exit(void);

#endif
